import java.io.*;
import java.util.StringTokenizer;

/**
 * This class generates a shell script from a list generated by
 * <CODE>ls -Rmp1 dir/</CODE>. Redirect the result of the command
 * to a file, and invoke this class on the file just obtained.
 */
public class ReadIteratedResults {

    public static void main(String args[]) {

        if(args.length < 1) {
            System.err.println("Sintaxe: java ResultCounter list");
        }

        int numColors=0;
        int highestColor=0;
        int maxSpillNumber=0;
        String maxSpillName = null;
        int numSpills=0;
        int totalNumTemp=0;
        int numTempRegisters=0;
        int totalMethods=0;
        int totalNumCoals=0;
        int totalNumMoves=0;
        int totalNumIterations=0;
        int maxIterationNumber=0;
        String methodName = null;
        int listOfColors[] = new int[40];

        for(int i = 0; i < listOfColors.length; i++)
            listOfColors[i] = 0;

        try {
            BufferedReader in = new BufferedReader(new FileReader(args[0]));

            while(true) {
                String line = in.readLine();
                if(line == null)
                    break;
                StringTokenizer st = new StringTokenizer(line);

                // Reads the method's name
                String mName  = st.nextToken();

                // reading the hash. This is just a legacy. Only read, and forget it.
                String boolValue = st.nextToken();

                // reading the number of temporaries used in the method
                int numTemporaries = Integer.parseInt(st.nextToken());
                totalNumTemp += numTemporaries;

                // reading the number of colors used in the allocation
                int numRegisters   = Integer.parseInt(st.nextToken());
                numColors += numRegisters;
                listOfColors[numRegisters]++;

                if(numRegisters > highestColor) {
                    highestColor = numRegisters;
                    numTempRegisters = numTemporaries;
                    methodName = mName;
                }

                // reading the number of spillings
                int numberOfSpills = Integer.parseInt(st.nextToken());
                if(numberOfSpills > 0) {
//                    System.out.print(numberOfSpills + "\t ");
                    numSpills += numberOfSpills;
                }
                if(numberOfSpills > maxSpillNumber) {
                    maxSpillNumber = numberOfSpills;
                    maxSpillName = mName;
                }

                // reading the number of coalescings
                int numCoalescing  = Integer.parseInt(st.nextToken());
                totalNumCoals += numCoalescing;

                // reading the number of move instructions
                int numMoveInstruc = Integer.parseInt(st.nextToken());
                totalNumMoves += numMoveInstruc;

                // reading the number of iteractions
                int numIterations = Integer.parseInt(st.nextToken());
                totalNumIterations += numIterations;
                if(numIterations > maxIterationNumber)
                    maxIterationNumber = numIterations;

                totalMethods++;
            }

            System.out.println("Results:");
            double tempPerMeth = ((double)totalNumTemp/(double)totalMethods);
            System.out.println("Average number of temporaries per method: " + tempPerMeth);
            double colorsAvg = ((double)numColors/(double)totalMethods);
            System.out.println("Average number of colors used: " + colorsAvg);
            System.out.println("Heightest color: " + highestColor + ", num temporaries: " + numTempRegisters + ", in " + methodName);
            double spillsPerMethod = ((double)numSpills/(double)totalMethods);
            System.out.println("Total number of spills: " + numSpills);
            System.out.println("Maximum number of spills: " + maxSpillNumber + " in " + maxSpillName);
            System.out.println("Average number of spills per method: " + spillsPerMethod);
            System.out.println("Total number of moves: " + totalNumMoves);
            System.out.println("Total number of coalescings: " + totalNumCoals);
            double iterAvg = ((double)totalNumIterations/(double)totalMethods);
            System.out.println("Iterations per method: " + iterAvg);
            System.out.println("Max number of iterations: " + maxIterationNumber);
            for(int i = 0; i <= highestColor; i++)
                System.out.println("Methods with " + i + " registers = " + listOfColors[i]);

        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
