package fernandoVisitorSet;

import FernandoGraph.*;

import joeq.Compiler.Quad.*;
import joeq.Compiler.Quad.SSA.*;
import joeq.Compiler.Dataflow.*;
import joeq.Main.Helper;
import joeq.Class.*;
import joeq.Util.Templates.*;

import java.io.IOException;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.HashSet;

public class FernandoVisitorSet implements jq_MethodVisitor {

    public void visitClassInitializer(jq_ClassInitializer m) {
//        System.out.println("VisitClassInitializer: " + m.toString());
    }

    public void visitInitializer(jq_Initializer m) {
//        System.out.println("VisitInitialier: " + m.toString());
    }

    public void visitInstanceMethod(jq_InstanceMethod m) {
//        System.out.println("VisitInstanceMethod: " + m.toString());
    }

    public void visitStaticMethod(jq_StaticMethod m) {
//        System.out.println("VisitStaticMethod: " + m.toString());
    }

    public void visitMethod(jq_Method m) {
        int choice = 0;

        switch(choice) {
            case 0:
                InterferenceGraphBuilder igb = new InterferenceGraphBuilder(true, true, 6, false);
                Helper.runPass(m, igb);
            break;
            case 1:
                LivenessAnalysisSolver_2 las = new LivenessAnalysisSolver_2();
                Helper.runPass(m, las);
            break;
            case 2:
                QuadPrinter qp = new QuadPrinter();
                Helper.runPass(m, qp);
            break;
            case 3:
                CFGPrinter cfgp = new CFGPrinter();
                Helper.runPass(m, cfgp);
            break;
        }
    }


    private class QuadPrinter extends QuadVisitor.EmptyVisitor {
        public void visitQuad(Quad q) {
            System.out.println(q.toString());
        }
    }


    private class LivenessAnalysisSolver_2 implements ControlFlowGraphVisitor {
        public void visitCFG(ControlFlowGraph cfg) {
            String methodName = cfg.getMethod().getName().toString();
            System.out.println("METHOD NAME: " + methodName + " --------------------------------------------- ");
            CFGSculptor sculp = new CFGSculptor(cfg);
            sculp.adaptCFG();

            // Print block information:
            System.out.println("Block information:");
            ListIterator.BasicBlock bbIterator = cfg.postOrderOnReverseGraphIterator();
            while(bbIterator.hasNext()) {
                BasicBlock bb = bbIterator.nextBasicBlock();
                System.out.println(bb.fullDump());
            }

            // Print liveness analysis information:
            System.out.println("Liveness analysis information:");
            LivenessAnalysis la = new LivenessAnalysis();
            Solver solver = new IterativeSolver();
            jwutil.graphs.EdgeGraph eg = new jwutil.graphs.EdgeGraph(new jwutil.graphs.ReverseGraph(cfg, java.util.Collections.singleton(cfg.exit())));
            solver.initialize(la, eg);
            solver.solve();
            Solver.dumpResults(cfg, solver);
        }
    }


    private class CFGPrinter implements ControlFlowGraphVisitor {
        public void visitCFG(ControlFlowGraph cfg) {
            ListIterator.BasicBlock bbIterator = cfg.postOrderOnReverseGraphIterator();
            while(bbIterator.hasNext()) {
                BasicBlock bb = bbIterator.nextBasicBlock();
                ListIterator.Quad qIter = bb.iterator();
                System.out.println("BEGIN");
                while(qIter.hasNext()) {
                    Quad quad = qIter.nextQuad();
                    System.out.println("\t" + quad);
                }
                System.out.println("END");
            }
        }
    }

}
